@@ -1,5 +1,7 @@
 from datetime import date, time, timedelta
 import json
+
+import pandas as pd
 import pytest
 
 import httpretty
@@ -443,6 +445,151 @@ def test_get_raw_apc_events():
     assert sent_req.querystring["date"] == ["2023-10-01"]
 
 
+@httpretty.activate
+def test_get_arrivals_departures_observations():
+    agency = "test_agency"
+    target_date = date(2023, 10, 1)
+    client = SwiftlyAPIClient(agency_key=agency, api_key="test_api_key")
+
+    url = client._format_url(client.ARRIVAL_DEPARTURE_OBSERVATIONS)
+
+    httpretty.register_uri(
+        httpretty.GET,
+        url,
+        responses=[
+            httpretty.Response(
+                body="""block_id,trip_id,route_id,route_short_name,direction_id,stop_id,stop_name,stop_code,trip_headsign,departure_vehicle_id,arrival_vehicle_id,stop_order,gtfs_stop_sequence,trip_start_time,is_last_stop_of_trip,is_first_stop_of_trip,adjustment_ids,adjustment_types,adjustment_effects,schedule_adherence_secs,scheduled_arrival_time,scheduled_departure_time,observed_arrival_time,observed_departure_time,is_schedule_adherence_stop,service_date
+401-1,t_5940767_b_83572_tn_6,401,401,0,2561195,Mall in Columbia,40000,Clary's Forest,,,0,1,14:00:00,false,true,,,,,14:00:00,14:00:00,,,true,2024-12-01
+401-1,t_5940768_b_83572_tn_10,401,401,1,2561170,Howard County Hospital East (Southbound),43019,Mall in Columbia,,,7,8,18:20:00,false,false,,,,,18:25:43,18:25:43,,,false,2024-12-01
+401-1,t_5940768_b_83572_tn_10,401,401,1,2561169,Howard County Hospital South (Southbound),43020,Mall in Columbia,,,8,9,18:20:00,false,false,,,,,18:26:00,18:26:00,,,false,2024-12-01
+402-1,t_5940784_b_83572_tn_1,402,402,0,2561156,Knoll Dr / Columbia Medical Plan,40669,Dobbin Center via Long Reach,,,5,6,09:30:00,false,false,,,,,09:38:00,09:38:00,,,true,2024-12-01
+402-1,t_5940784_b_83572_tn_2,402,402,0,2561156,Knoll Dr / Columbia Medical Plan,40669,Dobbin Center via Long Reach,,,5,6,10:30:00,false,false,,,,,10:38:00,10:38:00,,,true,2024-12-01
+402-1,t_5940784_b_83572_tn_3,402,402,0,2561156,Knoll Dr / Columbia Medical Plan,40669,Dobbin Center via Long Reach,,,5,6,11:30:00,false,false,,,,,11:38:00,11:38:00,,,true,2024-12-01
+                """,
+                status=200,
+            ),
+        ],
+    )
+
+    df = client.get_arrivals_departures_observations(target_date)
+
+    expected_df = pd.DataFrame(
+        {
+            "block_id": ["401-1", "401-1", "401-1", "402-1", "402-1", "402-1"],
+            "trip_id": [
+                "t_5940767_b_83572_tn_6",
+                "t_5940768_b_83572_tn_10",
+                "t_5940768_b_83572_tn_10",
+                "t_5940784_b_83572_tn_1",
+                "t_5940784_b_83572_tn_2",
+                "t_5940784_b_83572_tn_3",
+            ],
+            "route_id": [401, 401, 401, 402, 402, 402],
+            "route_short_name": [401, 401, 401, 402, 402, 402],
+            "direction_id": [0, 1, 1, 0, 0, 0],
+            "scheduled_arrival_time": [
+                "14:00:00",
+                "18:25:43",
+                "18:26:00",
+                "09:38:00",
+                "10:38:00",
+                "11:38:00",
+            ],
+            "service_date": [
+                "2024-12-01",
+                "2024-12-01",
+                "2024-12-01",
+                "2024-12-01",
+                "2024-12-01",
+                "2024-12-01",
+            ],
+        }
+    )
+
+    # just compare some specific columns
+    assert df[
+        [
+            "block_id",
+            "trip_id",
+            "route_id",
+            "route_short_name",
+            "direction_id",
+            "scheduled_arrival_time",
+            "service_date",
+        ]
+    ].equals(expected_df)
+
+    sent_req = httpretty.last_request()
+    assert sent_req.querystring["startDate"] == ["10-01-2023"]
+
+
+@httpretty.activate
+def test_get_arrivals_departures_observations_by_route():
+    agency = "test_agency"
+    target_date = date(2023, 10, 1)
+    client = SwiftlyAPIClient(agency_key=agency, api_key="test_api_key")
+
+    url = client._format_url(client.ARRIVAL_DEPARTURE_OBSERVATIONS)
+
+    httpretty.register_uri(
+        httpretty.GET,
+        url,
+        responses=[
+            httpretty.Response(
+                body="""block_id,trip_id,route_id,route_short_name,direction_id,stop_id,stop_name,stop_code,trip_headsign,departure_vehicle_id,arrival_vehicle_id,stop_order,gtfs_stop_sequence,trip_start_time,is_last_stop_of_trip,is_first_stop_of_trip,adjustment_ids,adjustment_types,adjustment_effects,schedule_adherence_secs,scheduled_arrival_time,scheduled_departure_time,observed_arrival_time,observed_departure_time,is_schedule_adherence_stop,service_date
+401-1,t_5940767_b_83572_tn_6,401,401,0,2561195,Mall in Columbia,40000,Clary's Forest,,,0,1,14:00:00,false,true,,,,,14:00:00,14:00:00,,,true,2024-12-01
+401-1,t_5940768_b_83572_tn_10,401,401,1,2561170,Howard County Hospital East (Southbound),43019,Mall in Columbia,,,7,8,18:20:00,false,false,,,,,18:25:43,18:25:43,,,false,2024-12-01
+401-1,t_5940768_b_83572_tn_10,401,401,1,2561169,Howard County Hospital South (Southbound),43020,Mall in Columbia,,,8,9,18:20:00,false,false,,,,,18:26:00,18:26:00,,,false,2024-12-01
+                """,
+                status=200,
+            ),
+        ],
+    )
+
+    df = client.get_arrivals_departures_observations(target_date, routes=["401", "404"])
+
+    expected_df = pd.DataFrame(
+        {
+            "block_id": ["401-1", "401-1", "401-1"],
+            "trip_id": [
+                "t_5940767_b_83572_tn_6",
+                "t_5940768_b_83572_tn_10",
+                "t_5940768_b_83572_tn_10",
+            ],
+            "route_id": [401, 401, 401],
+            "route_short_name": [401, 401, 401],
+            "direction_id": [0, 1, 1],
+            "scheduled_arrival_time": [
+                "14:00:00",
+                "18:25:43",
+                "18:26:00",
+            ],
+            "service_date": [
+                "2024-12-01",
+                "2024-12-01",
+                "2024-12-01",
+            ],
+        }
+    )
+
+    # just compare some specific columns
+    assert df[
+        [
+            "block_id",
+            "trip_id",
+            "route_id",
+            "route_short_name",
+            "direction_id",
+            "scheduled_arrival_time",
+            "service_date",
+        ]
+    ].equals(expected_df)
+
+    sent_req = httpretty.last_request()
+    assert sent_req.querystring["startDate"] == ["10-01-2023"]
+    assert sent_req.querystring["routes"] == ["401,404"]
+
+
 @httpretty.activate
 def test_get_arrivals_departures_observations_invalid_date_range():
     agency = "test_agency"
